/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.util.*;
import java.io.*;

public class ConstantFile extends Resource
{
  public static String PILRC_GENERATED;

	static int autogen=1000;

	Hashtable idMap;
	String parentDir;
	String file;
	String pckage;
	boolean autoGenerate=false;
	boolean pilrcGenerated=false;

	boolean dirty=false;

  public static void resetAutogen(int val)
  {
		autogen=val;
	}

  public ConstantFile()
  {
	}

	public ConstantFile(String parentDir,String file)
	{
		read(parentDir,file);
	}

  public void parse(StreamTokenizer st) throws IOException
  {
  }

  public void write(PrintStream ps)
  {
  }

  public int getConstantCount()
  {
		if (idMap==null)
		  return 0;
		return idMap.size();
	}

	public void addID(String id)
	{
    //System.out.println("addID("+id+") - "+autogen);
		setConstant(id,autogen++);
	}

  public boolean isDirty()
  {
		return dirty;
	}

	public void setDirty(boolean b)
	{
		dirty=b;
	}

  public boolean isPilRCGenerated()
  {
		return pilrcGenerated;
	}

	public void setPilRCGenerated(boolean b)
	{
		pilrcGenerated=b;
		if (pilrcGenerated)
		{
			file=PILRC_GENERATED;
			autoGenerate=true;
		}
	}

  public boolean isAutogenerating()
  {
		return autoGenerate;
	}

	public void setAutoGenerate(boolean b)
	{
		autoGenerate=b;
	}

  public void removeConstant(String id)
  {
		if (id==null||idMap==null)
		  return;
		idMap.remove(id);
	}

	public void setConstant(String id,int val)
	{
		setDirty(true);
		if (idMap==null)
			idMap=new Hashtable();
    if (val>=autogen)
      autogen=val+1;
		idMap.put(id,new Integer(val));
	}

  public boolean hasConstant(String id)
  {
		if (autoGenerate)
		  return true;
		return idMap!=null&&idMap.containsKey(id);
	}

  public Enumeration getAllConstants()
  {
		if (idMap==null)
		  return null;
		return idMap.keys();
	}

	public int getConstant(String id)
	{
		Integer i=(idMap==null?null:(Integer)idMap.get(id));
		if (i==null&&autoGenerate)
		{
      int con=autogen;
		  setConstant(id,con);
		  return con;
		}
		return i==null?-1:i.intValue();
	}

	public void read(String parentDir,String file)
	{
		setFilename(parentDir,file);
		read();
	}

	public void read()
	{
		if (isPilRCGenerated())
		  return;
		if (idMap!=null)
  		idMap.clear();
		StreamTokenizer st=null;
		try
		{
			boolean incFile=file.endsWith(".inc");
			FileReader fr=new FileReader(new File(parentDir,file));
			st=new StreamTokenizer(new BufferedReader(fr));
			st.wordChars('#','#');
			st.wordChars('_','_');
			int tok=0;
			while ((tok=st.nextToken())!=StreamTokenizer.TT_EOF)
			{
				if (tok==StreamTokenizer.TT_WORD)
				{
					if (st.sval.equals("#define"))
					{
						st.nextToken();
						String id=st.sval;
						st.nextToken();
						int val=(int)st.nval;
						setConstant(id,val);
					}
					else
	        if (st.lineno()==0&&st.sval.equals("package"))
	        {
						st.nextToken();
						pckage=st.sval;
					}
					else
					if (st.sval.equals("public"))
					{
						st.nextToken();
						if (!st.sval.equals("static"))
							continue;
						st.nextToken();
						if (!st.sval.equals("final"))
							continue;
						st.nextToken();
						if (!st.sval.equals("short"))
							continue;
						st.nextToken();
						String id=st.sval;
						st.nextToken(); // =
						st.nextToken();
						int val=(int)st.nval;
						setConstant(id,val);
					}
					else
					if (incFile)
					{
						String id=st.sval;
						st.nextToken();
						if (!st.sval.equals("equ"))
							continue;
						st.nextToken();
						int val=(int)st.nval;
						setConstant(id,val);
					}
				}
			}
			fr.close();
		}
		catch(FileNotFoundException e)
		{
			GeneralDialog gd=Preferences.getDialog();
			gd.setTitle("Missing File");
			gd.setText("The include file \""+file+"\" was not found.  Autogenerate it?");
			gd.setButtons(new String[]{"Yes","No"});
			autoGenerate=gd.showDialog()==0;
		}
		catch(Exception e)
		{
			GeneralDialog gd=Preferences.getDialog();
			gd.setTitle("Error");
			gd.setText("There was an error loading the include file \""+
				file+(st!=null?"\" at line "+st.lineno():"")+".");
			gd.setButtonOk();
			gd.show();
		}
		dirty=false;
	}

  public String getFile()
  {
		return file;
	}

  public void setFilename(String parentDir,String file)
  {
		this.parentDir=parentDir;
		this.file=file;
	}

	public void write()
	{
		if (isPilRCGenerated())
		  return;
		try
		{
			int dot=file.lastIndexOf('.');
			String ext=file.substring(dot+1,file.length()).toLowerCase();
			int type=ext.equals("java")?0:ext.equals("inc")?1:2;
			PrintStream ps=new PrintStream(new FileOutputStream(new File(parentDir,file)));
			if (type==0)
			{
				if (pckage!=null)
				{
				  ps.println("package "+pckage);
				  ps.println();
				}
			  ps.println("public class "+file.substring(0,dot));
			  ps.println("{");
			}
			for(Enumeration e=idMap.keys();e.hasMoreElements();)
			{
				String id=(String)e.nextElement();
				int val=((Integer)idMap.get(id)).intValue();
				switch(type)
				{
					case 0: ps.println("  public static final short "+id+"="+val); break;
					case 1: ps.println(id+" equ "+val);
					default: ps.println("#define "+id+" "+val);
				}
			}
			if (type==0)
			  ps.println("}");
			ps.close();
		}
		catch(Exception e){e.printStackTrace();}
	}
}