/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

public class Font
{
	public static String[] fontStrings;
	static Font[] fonts=new Font[8];
	static Vector customFonts=null;

  String file;
	Glyph[] glyphs=new Glyph[256];
	int fontHeight;
  int id;
  int index=-1;

  private Font(String file,int id)
  {
		this.id=id;
    parse(file);
  }

  public int getID()
  {
		return id;
	}


  public static void registerFont(int num,String file)
  {
		if (customFonts==null)
		  customFonts=new Vector();
		customFonts.addElement(new Font(file,num));
		String[] newFontStrings=new String[fontStrings.length+1];
	  System.arraycopy(fontStrings,0,newFontStrings,0,fontStrings.length);
	  fontStrings=newFontStrings;
	  int slash=file.lastIndexOf(File.separatorChar);
	  fontStrings[fontStrings.length-1]=file.substring(slash+1,file.length()-4);
	}

  public static String[] getFontStrings()
  {
		return fontStrings;
	}

  public int getIndex()
  {
		if (index==-1)
		{
			if (id<fonts.length)
				index=id;
		  else
			if (id>=128&&id<=255&&customFonts!=null)
			{
				index=fonts.length;
				for(Enumeration e=customFonts.elements();e.hasMoreElements();index++)
				{
					if (e.nextElement()==this)
						break;
				}
			}
		}
		return index;
	}

  public static Font getFontAtIndex(int num)
  {
		if (num<fonts.length)
		  return getFont(num);
		if (customFonts==null)
		  return null;
		return (Font)customFonts.elementAt(num-8);
	}

  public static Font getFont(int num)
  {
		if (num>=128&&num<=255&&customFonts!=null)
		{
			for(Enumeration e=customFonts.elements();e.hasMoreElements();)
			{
				Font f=(Font)e.nextElement();
				if (f.getID()==num)
				  return f;
			}
		}
		if (fonts[num]==null)
		{
			switch (num)
			{
				case 0: fonts[0]=new Font("fonts/stdfont.txt",0); break;
				case 1: fonts[1]=new Font("fonts/boldfont.txt",1); break;
				case 2: fonts[2]=new Font("fonts/bigfont.txt",2); break;
				case 3: fonts[3]=new Font("fonts/symbol.txt",3); break;
				case 4: fonts[4]=new Font("fonts/symbol11.txt",4); break;
				case 5: fonts[5]=new Font("fonts/symbol7.txt",5); break;
				case 6: fonts[6]=new Font("fonts/ledfont.txt",6); break;
				case 7: fonts[7]=new Font("fonts/bigbold.txt",7); break;
			}
		}
		return fonts[num];
	}

  public void drawChar(Graphics g,char ch,int x,int y)
  {
    int bgColor=Preferences.getIntPref(Bitmap.PREF_BGCOLOR,PilrcEdit.bgColor);
		Glyph gl=glyphs[ch];
		int col=g.getColor().getRGB();
		if (gl!=null)
		{
		  gl.setColor(col,bgColor);
		  g.drawImage(gl.pic,x,y,null);
		}
	}

  public void drawString(Graphics g,String text,int x,int y)
  {
		try
		{
		drawString(g,text,x,y,false);
	  }
	  catch(Exception e)
	  {
			System.out.println(text+":"+this+":"+x+":"+y);
			e.printStackTrace();
		}
	}

  public void drawString(Graphics g,String text,int x,int y,boolean invert)
  {
		if (text==null)
		  return;
		byte[] bytes=null;
    char[] chars=text.toCharArray();
		int col=g.getColor().getRGB();
		for(int i=0;i<chars.length;i++)
		{
      int index=chars[i];
      if (index<0||index>255)
      {
        if (bytes==null)
          bytes=text.getBytes();
        index=bytes[i]&0xFF;
      }
      //System.out.print("("+((int)chars[i])+":"+(bytes[i]&0xFF)+")");
			Glyph gl=glyphs[index];
			if (gl!=null)
			{
        int bgColor=Preferences.getIntPref(Bitmap.PREF_BGCOLOR,PilrcEdit.bgColor);

				if (invert)
				  gl.setColor(bgColor,col);
				else
  				gl.setColor(col,bgColor);
  		  if (gl.pic==null)
  		    System.out.println((int)gl.ch);
				g.drawImage(gl.pic,x,y,null);
				x+=gl.width;
			}
			else
			{
				g.drawRect(x,y+1,4,fontHeight-3);
				x+=5;
			}
		}
	}

  public int stringWidth(String text)
  {
		int width=0;
		byte[] bytes=null;
    char[] chars=text.toCharArray();
		for(int i=0;i<chars.length;i++)
		{
      int index=chars[i];
      if (index<0||index>255)
      {
        if (bytes==null)
          bytes=text.getBytes();
        index=bytes[i]&0xFF;
      }
			Glyph gl=glyphs[index];
			if (gl!=null)
				width+=gl.width;
		  else
		    width+=5;
		}
		return width;
	}

	public int getHeight()
	{
		return fontHeight;
	}

/*
  public static void writeCustomFonts(PrintStream ps)
  {
		if (customFonts==null)
		  return;
		for(Enumeration e=customFonts.elements();e.hasMoreElements();)
		{
			Font f=(Font)e.nextElement();
      ps.println("FONT ID "+fid+" FONTID "+fontID+" \""+fontfile+"\""
    }
	}
*/

  public void parse(String parentDir,String file)
  {
		this.file=file;
		parse(parentDir+file);
	}

  public void parse(String file)
  {
		try
		{
			InputStream is=Font.class.getResourceAsStream(file);
			if (is==null)
			  is=new FileInputStream(file);

			BufferedReader br=new BufferedReader(new InputStreamReader(is));

			String line=null;
			fontHeight=-1;
			Glyph glyph=null;
			String[] glyphLines=null;
			int lineCnt=0;
			while ((line=br.readLine())!=null)
			{
				if (line.startsWith("fRectHeight"))
				{
  				fontHeight=Integer.parseInt(line.substring(12));
  				glyphLines=new String[fontHeight];
				}
  			else
  			if (line.startsWith("GLYPH"))
				{
					String ch=line.substring(6);
					int glyphChar=0;
					if (ch.startsWith("'"))
						glyphChar=ch.charAt(1);
					else
						glyphChar=Integer.parseInt(ch);
					glyph=new Glyph();
					glyph.ch=(char)glyphChar;
					glyphs[glyphChar]=glyph;
					lineCnt=0;
				}
				else
				if (glyph!=null)
				{
					if (line.startsWith("OFFSET"))
					{
				    glyph.offset=Integer.parseInt(line.substring(7));
					}
					else
					if (line.equals("")&&lineCnt==0)
					{
						// skip blanks
					}
					else
				  if (line.equals("")||lineCnt==fontHeight)
					{
						if (fontHeight==-1)
						  fontHeight=lineCnt;
						glyph.setImage(glyphLines,lineCnt);
						glyph=null;
					}
					else
					{
						if (glyphLines==null)
						  glyphLines=new String[20];
						glyphLines[lineCnt++]=line;
					}
				}
			}
			if (glyph!=null)
			{
			  glyph.setImage(glyphLines,lineCnt);
			  glyph=null;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
  }
}

class Glyph
{
	char ch;
  int width;
  int height;
  int offset;
  Image pic;
  int col=Color.black.getRGB();
  int bcol=Preferences.getIntPref(Bitmap.PREF_BGCOLOR,PilrcEdit.bgColor); //PilrcEdit.bgColor.getRGB();
  int[] data;
  MemoryImageSource mis;

  public void setColor(int ncol,int nbcol)
  {
		if (col==ncol&&bcol==nbcol)
		  return;
		for(int i=0;i<data.length;i++)
		  if (data[i]==col)
		    data[i]=ncol;
		  else
		    data[i]=nbcol;
		col=ncol;
		bcol=nbcol;
		mis.newPixels(0,0,width,height);
	}

  public void setImage(String[] lines,int len)
  {
		height=len;
		width=lines[0].length();
		data=new int[width*height];
		int cnt=0;
		for(int i=0;i<height;i++)
		{
		  for(int j=0;j<width;j++)
		    data[cnt++]=(lines[i].charAt(j)=='#'?col:bcol);
		}

		try
		{
		mis=new MemoryImageSource(width, height,data, 0, width);
	  }
	  catch(Exception e)
	  {
			System.out.println(((int)ch)+">>"+width+","+height+","+data.length);
		}
		mis.setAnimated(true);
		mis.setFullBufferUpdates(true);
		pic=Toolkit.getDefaultToolkit().createImage(mis);
	}
}