/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import pilrcedit.*;
import java.awt.Graphics;
import java.awt.Color;
import java.io.*;
import java.util.Vector;

/**
 * Class representing a standard Palm button
 */
public class MenuBar extends FormObject
{
	private boolean boldFrame;
	private boolean noFrame;

  public String getObjectType()
  {
		return "Menu";
	}

	public void parse(StreamTokenizer st) throws IOException
	{
    /*
		readID(st);

		st.nextToken(); // begin
		if (!st.sval.equals("begin"))
		  throw new IllegalArgumentException("BEGIN Expected.");
		Vector menus=new Vector();
		st.nextToken();
		while(!st.sval.equals("end"))
		{
      Vector menuItems=new Vector();
      // PULLDOWN
      //readQuotedString(st);
		}
		String s=null;
		while((s=readQuotedString(st))!=null)
		{
		  listItemsVec.addElement(s);
		}
		int numItems=listItemsVec.size();
		listItems=new String[numItems];
		for(int i=0;i<listItems.length;i++)
			listItems[i]=(String)listItemsVec.elementAt(i);
    readID(st);
    readLocation(st);
    heightExp.setExpression("AUTO");
    parseOptions(st);
    if (font==null)
      setFont(0);
    height=0;
    evaluatePosition();
    */
	}

  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
		if (super.parseOption(st))
		  return true;
		if (st.sval.equals("boldframe"))
			boldFrame=true;
		else
		if (st.sval.equals("noframe"))
		  noFrame=true;
		else
		if (st.sval.equals("frame"));
		  // ignore
		else
		  return false;
		return true;
  }

	protected void writeOptions(StringBuffer sb)
	{
		super.writeOptions(sb);
		if (boldFrame)
		  sb.append(" BOLDFRAME");
		else
		if (noFrame)
		  sb.append(" NOFRAME");
	}

  public void getProperties(Properties prop)
  {
		super.getProperties(prop);
		prop.addProperty("Frame",new String[]{"None","Single","Bold"},noFrame?0:boldFrame?2:1);
	}

  public boolean setProperty(String name,String value,int index)
  {
		if (name.equals("Frame"))
	  {
			noFrame=index==0;
			boldFrame=index==2;
	  }
		else
		  return super.setProperty(name,value,index);
		evaluatePosition();
		return true;
	}

  public int autoWidth()
  {
		return super.autoWidth()+12;
	}

  public void paint(Graphics g)
  {
    //System.out.println(label+".paint("+boldFrame+":"+noFrame+")");
		int x1=x;
		int y1=y;
		int x2=x1+width-1;
		int y2=y1+height-1;
		g.setColor(usable?Color.black:Color.lightGray);
		String displayLabel=TextTools.unescape(label);
		Font font=getFont();
		font.drawString(g,displayLabel,x1+(width-font.stringWidth(displayLabel)+1)/2,
		  y1+(height-font.getHeight())/2);
		if (boldFrame)
		{
			g.drawLine(x1+3,y1-2,x2-3,y1-2);
			g.drawLine(x1+1,y1-1,x2-1,y1-1);
			g.drawLine(x1,y1,x1+3,y1);
			g.drawLine(x2-3,y1,x2,y1);
			g.drawLine(x1,y1+1,x1+1,y1+1);
			g.drawLine(x2-1,y1+1,x2,y1+1);
			g.drawLine(x1,y1+2,x1,y1+3);
			g.drawLine(x2,y1+2,x2,y1+3);
			g.drawLine(x1-1,y1+1,x1-1,y2-1);
			g.drawLine(x2+1,y1+1,x2+1,y2-1);
			g.drawLine(x1-2,y1+3,x1-2,y2-3);
			g.drawLine(x2+2,y1+3,x2+2,y2-3);
			g.drawLine(x1,y2-2,x1,y2-3);
			g.drawLine(x2,y2-2,x2,y2-3);
			g.drawLine(x2-1,y2-1,x2,y2-1);
			g.drawLine(x1,y2-1,x1+1,y2-1);
			g.drawLine(x2-3,y2,x2,y2);
			g.drawLine(x1,y2,x1+3,y2);
			g.drawLine(x1+1,y2+1,x2-1,y2+1);
			g.drawLine(x1+3,y2+2,x2-3,y2+2);
		}
		else
		if (!noFrame)
		{
			g.drawLine(x1,y1,x1+1,y1);
			g.drawLine(x1+2,y1-1,x2-2,y1-1);
			g.drawLine(x2-1,y1,x2,y1);
			g.drawLine(x1,y1+1,x1,y1+1);
			g.drawLine(x2,y1+1,x2,y1+1);
			g.drawLine(x1-1,y1+2,x1-1,y2-2);
			g.drawLine(x2+1,y1+2,x2+1,y2-2);
			g.drawLine(x1,y2-1,x1,y2-1);
			g.drawLine(x2,y2-1,x2,y2-1);
			g.drawLine(x1,y2,x1+1,y2);
			g.drawLine(x2-1,y2,x2,y2);
			g.drawLine(x1+2,y2+1,x2-2,y2+1);
		}
		g.setColor(Color.black);
  }
}