/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

/**
 * An extension of Expression that adds PREV* constants for positions relative
 * to another control.  Used as a base for both LocationExpression and DimensionExpression.
 */
public class PreviousExpression extends Expression
{
  public static final int PREVLEFT=100;
  public static final int PREVRIGHT=101;
  public static final int PREVWIDTH=102;
  public static final int PREVTOP=103;
  public static final int PREVBOTTOM=104;
  public static final int PREVHEIGHT=105;

  /** The previous object for relative positioning */
	FormObject prev;
  boolean isDependent=false;

  /**
   * Creates a new PreviousExpression.
   */
  public PreviousExpression()
  {
	}

  public PreviousExpression(PreviousExpression exp)
  {
		super(exp);
	}

  /**
   * Creates a new PreviousExpression.
   * @param obj the previous object for PREV* constants
   */
	public PreviousExpression(FormObject obj)
	{
		setPrevious(obj);
	}

  /**
   * Is this expression dependent on the previous object?
   */
  public boolean isDependent()
  {
		return isDependent;
	}

	/**
	 * Evaluates the expression string and returns the result (which is also available
	 * with getValue()).  Any parse errors will throw a IllegalArgumentException
	 * @return the integer value of the expression
	 * @throws java.lang.IllegalArgumentException
	 */
	public int evaluate()
	{
		isDependent=false;
		return super.evaluate();
	}

  /**
   * Sets the previous control (for use of PREV* constants)
   * @param obj the previous object in the list
   */
	public void setPrevious(FormObject obj)
	{
		prev=obj;
	}

  /**
   * Evaluate a string found in an expression that can't be parsed.
   * @param s the string to evaluate.
   */
	protected int evaluateString(String s)
	{
    int key=Keywords.get(this,s);
    if (key!=-1)
    {
      isDependent=true;
      if (prev==null)
        return 0;
    }
    switch(key)
    {
      case PREVLEFT: return prev.x;
      case PREVRIGHT: return prev.x+prev.width;
      case PREVTOP: return prev.y;
      case PREVBOTTOM: return prev.y+prev.height;
      case PREVWIDTH: return prev.width;
      case PREVHEIGHT: return prev.height;
      default: return super.evaluateString(s);
    }
	}
}
